@@ -32,6 +32,7 @@ class add : public expairseq
 {
 	GINAC_DECLARE_REGISTERED_CLASS(add, expairseq)
 	
+        friend class ex;
 	friend class mul;
 	friend class power;
 	
@@ -40,7 +41,7 @@ public:
 	add(const ex & lh, const ex & rh);
 	add(const exvector & v, bool hold=false);
 	add(const epvector & v);
-	add(const epvector & v, const ex & oc);
+	add(const epvector & v, const numeric & oc);
 //	add(std::unique_ptr<epvector> vp, const ex & oc);
 	
 	// functions overriding virtual functions from base classes
@@ -48,34 +49,32 @@ public:
 	unsigned precedence() const override {return 40;}
 	bool info(unsigned inf) const override;
 	bool is_polynomial(const ex & var) const override;
-	int degree(const ex & s) const override;
-	int ldegree(const ex & s) const override;
-	ex coeff(const ex & s, int n=1) const override;
+	numeric degree(const ex & s) const override;
+	numeric ldegree(const ex & s) const override;
+	ex coeff(const ex & s, const ex & n) const override;
 	ex eval(int level=0) const override;
-	ex evalm() const override;
 	ex series(const relational & r, int order, unsigned options = 0) const override;
-	ex normal(exmap & repl, exmap & rev_lookup, int level=0) const override;
+        void useries(flint_series_t& fp, int order) const override;
+	ex normal(exmap & repl, exmap & rev_lookup, int level=0, unsigned options = 0) const override;
 	numeric integer_content() const override;
 	ex smod(const numeric &xi) const override;
 	numeric max_coefficient() const override;
 	ex conjugate() const override;
 	ex real_part() const override;
 	ex imag_part() const override;
-	exvector get_free_indices() const override;
-	ex eval_ncmul(const exvector & v) const override;
 	const epvector & get_sorted_seq() const override;
 	ex lead_coeff() const;
 protected:
 	ex derivative(const symbol & s) const override;
 	unsigned return_type() const override;
 	tinfo_t return_type_tinfo() const override;
-	ex thisexpairseq(const epvector & v, const ex & oc, bool do_index_renaming = false) const override;
-	ex thisexpairseq(std::unique_ptr<epvector> vp, const ex & oc, bool do_index_renaming = false) const override;
+	ex thisexpairseq(const epvector & v, const numeric & oc, bool do_index_renaming = false) const override;
+	ex thisexpairseq(std::unique_ptr<epvector> vp, const numeric & oc, bool do_index_renaming = false) const override;
 	expair split_ex_to_pair(const ex & e) const override;
 	expair combine_ex_with_coeff_to_pair(const ex & e,
-	                                     const ex & c) const override;
+	                                     const numeric & c) const override;
 	expair combine_pair_with_coeff_to_pair(const expair & p,
-	                                       const ex & c) const override;
+	                                     const numeric & c) const override;
 	ex recombine_pair_to_ex(const expair & p) const override;
 	ex expand(unsigned options=0) const override;
 	ex eval_infinity(epvector::const_iterator infinity_iter) const;
@@ -85,8 +84,10 @@ protected:
 	void print_add(const print_context & c, unsigned level, bool latex) const;
 	void do_print(const print_context & c, unsigned level) const override;
 	void do_print_latex(const print_latex & c, unsigned level) const;
-	void do_print_csrc(const print_csrc & c, unsigned level) const;
 	void do_print_python_repr(const print_python_repr & c, unsigned level) const override;
+
+public:
+        ex combine_fractions() const;
 };
 
 } // namespace GiNaC
